import sys, os, subprocess, shlex
# By AWARE on RC: python3.6 and it happy this *local* version of dataclasses, but ver>3.6 should go with system-wide version
try:
    from dataclasses import dataclass
except:
    from dataclasses_local import dataclass
from pathlib import Path

from .remote_interfaces import RemoterInterface
from .status import Status


@dataclass
class Scp_wrap(RemoterInterface):
    __version__ = "0.0.1"
    internal_name: str  = "[SCP_WRAP]"
    verbose      : bool      = False
    debug        : bool      = False

    host         : str  = "192.168.2.52"
    port         : int  = 22
    username     : str  = "nucky"
    psw          : str  = ""

    remote_home  : Path = None
    local_home   : Path = None

    def __post_init__(self):
        if self.local_home is None: 
            self.local_home = Path.home()

        # Update the host key
        cmd: str = f"ssh -p {self.port} {self.username}@{self.host} -o StrictHostKeyChecking=no"
        self._execute(cmd=cmd, msg="Initialization")


    def _path_checker(self, _path:Path = Path("~/some/path/2/file.dat"), local:bool = False, remote:bool = False) -> Path:
        if local and remote: raise ValueError

        if local:  correct_home: Path = self.local_home
        if remote: correct_home: Path = self.remote_home

        if _path.home() != correct_home:
            _path_without_home: Path = Path(str(_path)[len(str(_path.home()))+1:])
            _path = correct_home / _path_without_home

        return _path

    def connect(self) -> Status:
        s: Status = self.execute(cmd=f"ls")
        if self.remote_home is None:
            _s: Status = self.execute(cmd=f"pwd")
            output: str  = _s.get_output_msg()
            self.remote_home = output.strip()

        if self.debug: print("Remote home: ", self.remote_home)

        # No need to provide any output at connection stage
        s.message = ""        
        return s

    def is_connected(self) -> bool:
        s:Status = self.execute(cmd=f"ls")
        return s.is_ok()
    
    '''
    It will be always successfully disconnected because it exist at each execution
    '''
    def disconnect(self) -> Status:
        return Status("OK")
    
    '''
    TODO: Verification?
    '''
    def upload(self, src_fname: Path = None, dst_fname: Path = None):
        src_fname = self._path_checker(src_fname, local=True)
        dst_fname = self._path_checker(dst_fname, remote=True)
        cmd = f"scp -P {self.port} -r {str(src_fname)} {self.username}@{self.host}:{dst_fname}"
        return self._execute(cmd=cmd, msg="Uploading")

    '''
    TODO: Verification?
    '''
    def download(self, src_fname: Path = None, dst_fname: Path = None):
        src_fname = self._path_checker(src_fname, remote=True)
        dst_fname = self._path_checker(dst_fname, local=True)
        cmd = f"scp -P {self.port} -r {self.username}@{self.host}:{str(src_fname)} {dst_fname}"
        return self._execute(cmd=cmd, msg="Downloading")

    def execute(self, cmd:str):
        cmd = f"ssh -p {self.port} {self.username}@{self.host} -oPreferredAuthentications=publickey {cmd}"
        return self._execute(cmd=cmd, msg="Executing")
    
    def _execute(self, cmd: str, msg: str) -> Status:
        if self.debug: print(f"{self.internal_name} {msg}... [{cmd}]")
        if self.debug: print(f"Working:::{msg}>> '{cmd}'")
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        
        stderr = output.stderr.strip()
        if stderr: return Status("FAIL", message=stderr)
        return Status("OK", message=output.stdout.strip())


    '''
    TODO
    '''
    def exists(self, path: Path) -> bool:
        return False